@@ -79,16 +79,8 @@
 #include <QToolTip>
 #include <QScrollArea>
 #include <QCollator>
-#include <QPrintDialog>
-#include <QPrinter>
 #include <QTemporaryFile>
 
-#ifdef Q_OS_WIN
-#include <qt_windows.h>
-#else
-#include "fileprinter.h"
-#endif
-
 #ifdef QZ_WS_X11
 #include <QX11Info>
 #include <xcb/xcb.h>
@@ -172,6 +164,7 @@ void BrowserWindow::postLaunch()
 
     case MainApplication::OpenHomePage:
     case MainApplication::RestoreSession:
+    case MainApplication::SelectSession:
         startUrl = m_homepage;
         break;
 
@@ -186,7 +179,7 @@ void BrowserWindow::postLaunch()
             startUrl.clear();
             m_tabWidget->addView(QUrl("qupzilla:restore"), Qz::NT_CleanSelectedTabAtTheEnd);
         }
-        else if (mApp->afterLaunch() == MainApplication::RestoreSession && mApp->restoreManager()) {
+        else if ((mApp->afterLaunch() == MainApplication::SelectSession || mApp->afterLaunch() == MainApplication::RestoreSession) && mApp->restoreManager()) {
             addTab = !mApp->restoreSession(this, mApp->restoreManager()->restoreData());
         }
         break;
@@ -348,26 +341,26 @@ void BrowserWindow::setupUi()
 
 void BrowserWindow::setupMenu()
 {
-#ifdef Q_OS_MAC
+#ifdef Q_OS_MACOS
     static MainMenu* macMainMenu = 0;
 
     if (!macMainMenu) {
         macMainMenu = new MainMenu(this, 0);
         macMainMenu->initMenuBar(new QMenuBar(0));
+        connect(mApp, SIGNAL(activeWindowChanged(BrowserWindow*)), macMainMenu, SLOT(setWindow(BrowserWindow*)));
     }
     else {
         macMainMenu->setWindow(this);
     }
 
     m_mainMenu = macMainMenu;
-    m_mainMenu->initSuperMenu(m_superMenu);
 #else
     setMenuBar(new MenuBar(this));
 
     m_mainMenu = new MainMenu(this, this);
     m_mainMenu->initMenuBar(menuBar());
-    m_mainMenu->initSuperMenu(m_superMenu);
 #endif
+    m_mainMenu->initSuperMenu(m_superMenu);
 
     // Setup other shortcuts
     QShortcut* reloadBypassCacheAction = new QShortcut(QKeySequence(QSL("Ctrl+F5")), this);
@@ -479,7 +472,7 @@ void BrowserWindow::loadSettings()
     m_bookmarksToolbar->setVisible(showBookmarksToolbar);
     m_navigationToolbar->setVisible(showNavigationToolbar);
 
-#ifndef Q_OS_MAC
+#ifndef Q_OS_MACOS
     menuBar()->setVisible(!isFullScreen() && showMenuBar);
 #endif
 
@@ -615,48 +608,7 @@ void BrowserWindow::changeEncoding()
 
 void BrowserWindow::printPage()
 {
-    QPrintDialog* dialog = new QPrintDialog(this);
-    dialog->setOptions(QAbstractPrintDialog::PrintToFile | QAbstractPrintDialog::PrintShowPageSize);
-#ifndef Q_OS_WIN
-    dialog->setOption(QAbstractPrintDialog::PrintPageRange);
-    if (Qz::FilePrinter::cupsAvailable()) {
-        dialog->setOption(QAbstractPrintDialog::PrintCollateCopies);
-    }
-#endif
-    dialog->printer()->setCreator(tr("QupZilla %1 (%2)").arg(Qz::VERSION, Qz::WWWADDRESS));
-    dialog->printer()->setDocName(QzTools::getFileNameFromUrl(weView()->url()));
-
-    if (dialog->exec() == QDialog::Accepted) {
-        if (dialog->printer()->outputFormat() == QPrinter::PdfFormat) {
-            weView()->page()->printToPdf(dialog->printer()->outputFileName(), dialog->printer()->pageLayout());
-            delete dialog;
-        } else {
-            weView()->page()->printToPdf([=](const QByteArray &data) {
-                if (!data.isEmpty()) {
-                    QTemporaryFile tempFile(QDir::tempPath() + QSL("/QupZillaPrintXXXXXX.pdf"));
-                    tempFile.setAutoRemove(false);
-                    if (tempFile.open()) {
-                        qint64 bytesWritten = tempFile.write(data);
-                        tempFile.close();
-                        if (bytesWritten == data.size()) {
-#ifdef Q_OS_WIN
-                            QString printerName = '"' + dialog->printer()->printerName() + '"';
-                            // This may bring up a PDF viewer window, and even keep it open, but it is the best we can do without adding third-party dependencies.
-                            // lpr is not installed by default on Windows, and it also can only print PDF if the printer handles it in hardware.
-                            ShellExecuteW((HWND)winId(), L"printto", (LPCWSTR)tempFile.fileName().utf16(), (LPCWSTR)printerName.utf16(), NULL, SW_HIDE);
-                            // TODO: When can we delete the file?
-#else
-                            Qz::FilePrinter::printFile(dialog->printer(), tempFile.fileName(), Qz::FilePrinter::SystemDeletesFiles, Qz::FilePrinter::SystemSelectsPages);
-#endif
-                        } else {
-                            tempFile.remove();
-                        }
-                    }
-                }
-                delete dialog;
-            }, dialog->printer()->pageLayout());
-        }
-    }
+    weView()->printPage();
 }
 
 void BrowserWindow::bookmarkPage()
@@ -731,6 +683,15 @@ void BrowserWindow::showSource(WebView *view)
     view->showSource();
 }
 
+void BrowserWindow::showNormal()
+{
+    if (m_normalWindowState & Qt::WindowMaximized) {
+        QMainWindow::showMaximized();
+    } else {
+        QMainWindow::showNormal();
+    }
+}
+
 SideBar* BrowserWindow::addSideBar()
 {
     if (m_sideBar) {
@@ -758,7 +719,7 @@ void BrowserWindow::saveSideBarWidth()
 
 void BrowserWindow::toggleShowMenubar()
 {
-#ifdef Q_OS_MAC
+#ifdef Q_OS_MACOS
     // We use one shared global menubar on Mac that can't be hidden
     return;
 #endif
@@ -812,7 +773,7 @@ void BrowserWindow::toggleShowNavigationToolbar()
 
     Settings().setValue("Browser-View-Settings/showNavigationToolbar", m_navigationToolbar->isVisible());
 
-#ifndef Q_OS_MAC
+#ifndef Q_OS_MACOS
     // Make sure we show Menu Bar when Navigation Toolbar is hidden
     if (!m_navigationToolbar->isVisible() && !menuBar()->isVisible()) {
         toggleShowMenubar();
@@ -834,15 +795,22 @@ void BrowserWindow::toggleFullScreen()
     }
 
     if (isFullScreen())
-        setWindowState(windowState() & ~Qt::WindowFullScreen);
+        showNormal();
     else
-        setWindowState(windowState() | Qt::WindowFullScreen);
+        showFullScreen();
 }
 
-void BrowserWindow::enterHtmlFullScreen()
+void BrowserWindow::toggleHtmlFullScreen(bool enable)
 {
-    showFullScreen();
-    m_isHtmlFullScreen = true;
+    if (enable)
+        showFullScreen();
+    else
+        showNormal();
+
+    if (m_sideBar)
+        m_sideBar.data()->setHidden(enable);
+
+    m_isHtmlFullScreen = enable;
 }
 
 void BrowserWindow::showWebInspector()
@@ -926,7 +894,7 @@ void BrowserWindow::createToolbarsMenu(QMenu* menu)
 
     QAction* action;
 
-#ifndef Q_OS_MAC
+#ifndef Q_OS_MACOS
     action = menu->addAction(tr("&Menu Bar"), this, SLOT(toggleShowMenubar()));
     action->setCheckable(true);
     action->setChecked(menuBar()->isVisible());
@@ -1094,15 +1062,13 @@ void BrowserWindow::hideNavigationSlot()
 bool BrowserWindow::event(QEvent* event)
 {
     switch (event->type()) {
-    case QEvent::WindowStateChange: {
-        QWindowStateChangeEvent* ev = static_cast<QWindowStateChangeEvent*>(event);
-
-        if (!(ev->oldState() & Qt::WindowFullScreen) && windowState() & Qt::WindowFullScreen) {
+    case QEvent::WindowStateChange:
+        if (!(m_oldWindowState & Qt::WindowFullScreen) && windowState() & Qt::WindowFullScreen) {
             // Enter fullscreen
-            m_windowStates = ev->oldState();
+            m_normalWindowState = m_oldWindowState;
 
             m_statusBarVisible = statusBar()->isVisible();
-#ifndef Q_OS_MAC
+#ifndef Q_OS_MACOS
             m_menuBarVisible = menuBar()->isVisible();
             menuBar()->hide();
 #endif
@@ -1111,10 +1077,10 @@ bool BrowserWindow::event(QEvent* event)
             m_navigationContainer->hide();
             m_navigationToolbar->buttonExitFullscreen()->show();
         }
-        else if (ev->oldState() & Qt::WindowFullScreen && !(windowState() & Qt::WindowFullScreen)) {
+        else if (m_oldWindowState & Qt::WindowFullScreen && !(windowState() & Qt::WindowFullScreen)) {
             // Leave fullscreen
             statusBar()->setVisible(m_statusBarVisible);
-#ifndef Q_OS_MAC
+#ifndef Q_OS_MACOS
             menuBar()->setVisible(m_menuBarVisible);
 #endif
 
@@ -1122,15 +1088,14 @@ bool BrowserWindow::event(QEvent* event)
             m_navigationToolbar->setSuperMenuVisible(!m_menuBarVisible);
             m_navigationToolbar->buttonExitFullscreen()->hide();
             m_isHtmlFullScreen = false;
-
-            setWindowState(m_windowStates);
         }
 
         if (m_hideNavigationTimer) {
             m_hideNavigationTimer->stop();
         }
+
+        m_oldWindowState = windowState();
         break;
-    }
 
     default:
         break;
@@ -1177,6 +1142,7 @@ void BrowserWindow::keyPressEvent(QKeyEvent* event)
         }
         break;
 
+    case Qt::Key_Reload:
     case Qt::Key_Refresh:
         if (view) {
             view->reload();
@@ -1352,6 +1318,18 @@ void BrowserWindow::keyReleaseEvent(QKeyEvent* event)
         return;
     }
 
+    switch (event->key()) {
+    case Qt::Key_F:
+        if (event->modifiers() == Qt::ControlModifier) {
+            action(QSL("Edit/Find"))->trigger();
+            event->accept();
+        }
+        break;
+
+    default:
+        break;
+    }
+
     QMainWindow::keyReleaseEvent(event);
 }
 
@@ -1364,18 +1342,18 @@ void BrowserWindow::closeEvent(QCloseEvent* event)
     Settings settings;
     bool askOnClose = settings.value("Browser-Tabs-Settings/AskOnClosing", true).toBool();
 
-    if (mApp->afterLaunch() == MainApplication::RestoreSession && mApp->windowCount() == 1) {
+    if ((mApp->afterLaunch() == MainApplication::SelectSession || mApp->afterLaunch() == MainApplication::RestoreSession) && mApp->windowCount() == 1) {
         askOnClose = false;
     }
 
     if (askOnClose && m_tabWidget->normalTabsCount() > 1) {
-        CheckBoxDialog dialog(QDialogButtonBox::Yes | QDialogButtonBox::No, this);
+        CheckBoxDialog dialog(QMessageBox::Yes | QMessageBox::No, this);
         dialog.setText(tr("There are still %n open tabs and your session won't be stored. \nAre you sure you want to close this window?", "", m_tabWidget->count()));
         dialog.setCheckBoxText(tr("Don't ask again"));
         dialog.setWindowTitle(tr("There are still open tabs"));
-        dialog.setIcon(IconProvider::standardIcon(QStyle::SP_MessageBoxWarning));
+        dialog.setIcon(QMessageBox::Warning);
 
-        if (dialog.exec() != QDialog::Accepted) {
+        if (dialog.exec() != QMessageBox::Yes) {
             event->ignore();
             return;
         }
@@ -1387,7 +1365,7 @@ void BrowserWindow::closeEvent(QCloseEvent* event)
 
     saveSettings();
 
-    #ifndef Q_OS_MAC
+    #ifndef Q_OS_MACOS
         if (mApp->windowCount() == 1)
             mApp->quitApplication();
     #endif
@@ -1397,7 +1375,7 @@ void BrowserWindow::closeEvent(QCloseEvent* event)
 
 void BrowserWindow::closeWindow()
 {
-#ifdef Q_OS_MAC
+#ifdef Q_OS_MACOS
     close();
     return;
 #endif
