@@ -1,6 +1,6 @@
 /* ============================================================
-* QupZilla - WebKit based browser
-* Copyright (C) 2010-2016  David Rosca <nowrep@gmail.com>
+* QupZilla - Qt web browser
+* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -21,15 +21,17 @@
 #include "mainapplication.h"
 #include "downloadoptionsdialog.h"
 #include "downloaditem.h"
-#include "ecwin7.h"
 #include "networkmanager.h"
-#include "qtwin.h"
 #include "desktopnotificationsfactory.h"
 #include "qztools.h"
 #include "webpage.h"
 #include "webview.h"
 #include "settings.h"
 #include "datapaths.h"
+#include "tabwidget.h"
+#include "tabbedwebview.h"
+#include "tabbar.h"
+#include "locationbar.h"
 
 #include <QMessageBox>
 #include <QCloseEvent>
@@ -39,6 +41,12 @@
 #include <QWebEngineHistory>
 #include <QWebEngineDownloadItem>
 
+#ifdef Q_OS_WIN
+#include <QtWin>
+#include <QWinTaskbarButton>
+#include <QWinTaskbarProgress>
+#endif
+
 DownloadManager::DownloadManager(QWidget* parent)
     : QWidget(parent)
     , ui(new Ui::DownloadManager)
@@ -49,7 +57,7 @@ DownloadManager::DownloadManager(QWidget* parent)
     ui->setupUi(this);
 #ifdef Q_OS_WIN
     if (QtWin::isCompositionEnabled()) {
-        QtWin::extendFrameIntoClientArea(this);
+        QtWin::extendFrameIntoClientArea(this, -1, -1, -1, -1);
     }
 #endif
     ui->clearButton->setIcon(QIcon::fromTheme("edit-clear"));
@@ -63,12 +71,6 @@ DownloadManager::DownloadManager(QWidget* parent)
     loadSettings();
 
     QzTools::setWmClass("Download Manager", this);
-
-#ifdef W7TASKBAR
-    if (QtWin::isRunningWindows7()) {
-        win7.init(QtWin::hwndOfWidget(this));
-    }
-#endif
 }
 
 void DownloadManager::loadSettings()
@@ -115,6 +117,45 @@ void DownloadManager::keyPressEvent(QKeyEvent* e)
     QWidget::keyPressEvent(e);
 }
 
+void DownloadManager::closeDownloadTab(const QUrl &url) const
+{
+    // Attempt to close empty tab that was opened only for loading the download url
+    auto testWebView = [](TabbedWebView *view, const QUrl &url) {
+        if (view->browserWindow()->tabWidget()->tabBar()->normalTabsCount() < 2) {
+            return false;
+        }
+        WebPage *page = view->page();
+        if (page->history()->count() != 0) {
+            return false;
+        }
+        if (page->url() != QUrl()) {
+            return false;
+        }
+        QUrl tabUrl = page->requestedUrl();
+        if (tabUrl.isEmpty()) {
+            tabUrl = QUrl(view->webTab()->locationBar()->text());
+        }
+        return tabUrl.host() == url.host();
+    };
+
+    if (testWebView(mApp->getWindow()->weView(), url)) {
+        mApp->getWindow()->weView()->closeView();
+        return;
+    }
+
+    const auto windows = mApp->windows();
+    for (auto *window : windows) {
+        const auto tabs = window->tabWidget()->allTabs();
+        for (auto *tab : tabs) {
+            auto *view = tab->webView();
+            if (testWebView(view, url)) {
+                view->closeView();
+                return;
+            }
+        }
+    }
+}
+
 void DownloadManager::startExternalManager(const QUrl &url)
 {
     QString arguments = m_externalArguments;
@@ -124,15 +165,6 @@ void DownloadManager::startExternalManager(const QUrl &url)
     m_lastDownloadOption = ExternalManager;
 }
 
-#ifdef W7TASKBAR
-bool DownloadManager::nativeEvent(const QByteArray &eventType, void* _message, long* result)
-{
-    Q_UNUSED(eventType)
-    MSG* message = static_cast<MSG*>(_message);
-    return win7.winEvent(message, result);
-}
-#endif
-
 void DownloadManager::timerEvent(QTimerEvent* e)
 {
     QVector<QTime> remTimes;
@@ -143,11 +175,16 @@ void DownloadManager::timerEvent(QTimerEvent* e)
         if (!ui->list->count()) {
             ui->speedLabel->clear();
             setWindowTitle(tr("Download Manager"));
+#ifdef Q_OS_WIN
+            if (m_taskbarButton) {
+                m_taskbarButton->progress()->hide();
+            }
+#endif
             return;
         }
         for (int i = 0; i < ui->list->count(); i++) {
             DownloadItem* downItem = qobject_cast<DownloadItem*>(ui->list->itemWidget(ui->list->item(i)));
-            if (!downItem || (downItem && downItem->isCancelled()) || !downItem->isDownloading()) {
+            if (!downItem || downItem->isCancelled() || !downItem->isDownloading()) {
                 continue;
             }
             progresses.append(downItem->progress());
@@ -176,14 +213,16 @@ void DownloadManager::timerEvent(QTimerEvent* e)
             speed += spee;
         }
 
+#ifndef Q_OS_WIN
         ui->speedLabel->setText(tr("%1% of %2 files (%3) %4 remaining").arg(QString::number(progress), QString::number(progresses.count()),
                                 DownloadItem::currentSpeedToString(speed),
                                 DownloadItem::remaingTimeToString(remaining)));
+#endif
         setWindowTitle(tr("%1% - Download Manager").arg(progress));
-#ifdef W7TASKBAR
-        if (QtWin::isRunningWindows7()) {
-            win7.setProgressValue(progress, 100);
-            win7.setProgressState(win7.Normal);
+#ifdef Q_OS_WIN
+        if (m_taskbarButton) {
+            m_taskbarButton->progress()->show();
+            m_taskbarButton->progress()->setValue(progress);
         }
 #endif
     }
@@ -209,29 +248,49 @@ void DownloadManager::clearList()
 
 void DownloadManager::download(QWebEngineDownloadItem *downloadItem)
 {
+    closeDownloadTab(downloadItem->url());
+
     QString downloadPath;
     bool openFile = false;
 
     QString fileName = QFileInfo(downloadItem->path()).fileName();
     fileName = QUrl::fromPercentEncoding(fileName.toUtf8());
+    // Filename may have been percent encoded and actually containing path
+    fileName = QFileInfo(fileName).fileName();
+
+    const bool forceAsk = downloadItem->savePageFormat() != QWebEngineDownloadItem::UnknownSaveFormat
+            || downloadItem->type() == QWebEngineDownloadItem::UserRequested;
 
     if (m_useExternalManager) {
         startExternalManager(downloadItem->url());
-    } else if (m_downloadPath.isEmpty()) {
-        // Ask what to do
-        DownloadOptionsDialog optionsDialog(fileName, downloadItem->url(), mApp->activeWindow());
-        optionsDialog.showExternalManagerOption(m_useExternalManager);
-        optionsDialog.setLastDownloadOption(m_lastDownloadOption);
+    } else if (forceAsk || m_downloadPath.isEmpty()) {
+        enum Result { Open = 1, Save = 2, ExternalManager = 3, SavePage = 4, Unknown = 0 };
+        Result result = Unknown;
 
-        switch (optionsDialog.exec()) {
-        case 1: // Open
+        if (downloadItem->savePageFormat() != QWebEngineDownloadItem::UnknownSaveFormat) {
+            // Save Page requested
+            result = SavePage;
+        } else if (downloadItem->type() == QWebEngineDownloadItem::UserRequested) {
+            // Save x as... requested
+            result = Save;
+        } else {
+            // Ask what to do
+            DownloadOptionsDialog optionsDialog(fileName, downloadItem, mApp->activeWindow());
+            optionsDialog.showExternalManagerOption(m_useExternalManager);
+            optionsDialog.setLastDownloadOption(m_lastDownloadOption);
+            result = Result(optionsDialog.exec());
+        }
+
+        switch (result) {
+        case Open:
             openFile = true;
             downloadPath = QzTools::ensureUniqueFilename(DataPaths::path(DataPaths::Temp) + QLatin1Char('/') + fileName);
             m_lastDownloadOption = OpenFile;
             break;
 
-        case 2: // Save
+        case Save:
             downloadPath = QFileDialog::getSaveFileName(mApp->activeWindow(), tr("Save file as..."), m_lastDownloadPath + QLatin1Char('/') + fileName);
+
             if (!downloadPath.isEmpty()) {
                 m_lastDownloadPath = QFileInfo(downloadPath).absolutePath();
                 Settings().setValue(QSL("DownloadManager/lastDownloadPath"), m_lastDownloadPath);
@@ -239,7 +298,40 @@ void DownloadManager::download(QWebEngineDownloadItem *downloadItem)
             }
             break;
 
-        case 3: // External manager
+        case SavePage: {
+            const QString mhtml = tr("MIME HTML Archive (*.mhtml)");
+            const QString htmlSingle = tr("HTML Page, single (*.html)");
+            const QString htmlComplete = tr("HTML Page, complete (*.html)");
+            const QString filter = QStringLiteral("%1;;%2;;%3").arg(mhtml, htmlSingle, htmlComplete);
+
+            QString selectedFilter;
+            downloadPath = QFileDialog::getSaveFileName(mApp->activeWindow(), tr("Save page as..."),
+                                                        m_lastDownloadPath + QLatin1Char('/') + fileName,
+                                                        filter, &selectedFilter);
+
+            if (!downloadPath.isEmpty()) {
+                m_lastDownloadPath = QFileInfo(downloadPath).absolutePath();
+                Settings().setValue(QSL("DownloadManager/lastDownloadPath"), m_lastDownloadPath);
+                m_lastDownloadOption = SaveFile;
+
+                QWebEngineDownloadItem::SavePageFormat format = QWebEngineDownloadItem::UnknownSaveFormat;
+
+                if (selectedFilter == mhtml) {
+                    format = QWebEngineDownloadItem::MimeHtmlSaveFormat;
+                } else if (selectedFilter == htmlSingle) {
+                    format = QWebEngineDownloadItem::SingleHtmlSaveFormat;
+                } else if (selectedFilter == htmlComplete) {
+                    format = QWebEngineDownloadItem::CompleteHtmlSaveFormat;
+                }
+
+                if (format != QWebEngineDownloadItem::UnknownSaveFormat) {
+                    downloadItem->setSavePageFormat(format);
+                }
+            }
+            break;
+        }
+
+        case ExternalManager:
             startExternalManager(downloadItem->url());
             // fallthrough
 
@@ -279,7 +371,7 @@ void DownloadManager::downloadFinished(bool success)
     bool downloadingAllFilesFinished = true;
     for (int i = 0; i < ui->list->count(); i++) {
         DownloadItem* downItem = qobject_cast<DownloadItem*>(ui->list->itemWidget(ui->list->item(i)));
-        if (!downItem || (downItem && downItem->isCancelled()) || !downItem->isDownloading()) {
+        if (!downItem || downItem->isCancelled() || !downItem->isDownloading()) {
             continue;
         }
         downloadingAllFilesFinished = false;
@@ -287,7 +379,7 @@ void DownloadManager::downloadFinished(bool success)
 
     if (downloadingAllFilesFinished) {
         if (success && qApp->activeWindow() != this) {
-            mApp->desktopNotifications()->showNotification(QIcon::fromTheme(QSL("download"), QIcon(":icons/notifications/download.png")).pixmap(48), tr("Download Finished"), tr("All files have been successfully downloaded."));
+            mApp->desktopNotifications()->showNotification(QIcon::fromTheme(QSL("download"), QIcon(QSL(":icons/other/download.svg"))).pixmap(48), tr("Download Finished"), tr("All files have been successfully downloaded."));
             if (!m_closeOnFinish) {
                 raise();
                 activateWindow();
@@ -295,10 +387,9 @@ void DownloadManager::downloadFinished(bool success)
         }
         ui->speedLabel->clear();
         setWindowTitle(tr("Download Manager"));
-#ifdef W7TASKBAR
-        if (QtWin::isRunningWindows7()) {
-            win7.setProgressValue(0, 100);
-            win7.setProgressState(win7.NoProgress);
+#ifdef Q_OS_WIN
+        if (m_taskbarButton) {
+            m_taskbarButton->progress()->hide();
         }
 #endif
         if (m_closeOnFinish) {
@@ -307,6 +398,18 @@ void DownloadManager::downloadFinished(bool success)
     }
 }
 
+void DownloadManager::showEvent(QShowEvent *event)
+{
+    QWidget::showEvent(event);
+#ifdef Q_OS_WIN
+    if (!m_taskbarButton) {
+        m_taskbarButton = new QWinTaskbarButton(this);
+    }
+    m_taskbarButton->setWindow(windowHandle());
+    m_taskbarButton->progress()->setRange(0, 100);
+#endif
+}
+
 void DownloadManager::deleteItem(DownloadItem* item)
 {
     if (item && !item->isDownloading()) {
