@@ -95,6 +95,7 @@ void LocationCompleter::complete(const QString &string)
 
 void LocationCompleter::showMostVisited()
 {
+    m_locationBar->setFocus();
     complete(QString());
 }
 
@@ -105,7 +106,7 @@ void LocationCompleter::refreshJobFinished()
 
     // Don't show results of older jobs
     // Also don't open the popup again when it was already closed
-    if (job->timestamp() > m_lastRefreshTimestamp && !m_popupClosed) {
+    if (!job->isCanceled() && job->timestamp() > m_lastRefreshTimestamp && !m_popupClosed) {
         s_model->setCompletions(job->completions());
         m_lastRefreshTimestamp = job->timestamp();
 
@@ -166,8 +167,9 @@ void LocationCompleter::addSuggestions(const QStringList &suggestions)
     }
 
     s_model->addCompletions(items);
-    adjustPopupSize();
     m_oldSuggestions = suggestions;
+
+    showPopup();
 }
 
 void LocationCompleter::currentChanged(const QModelIndex &index)
@@ -295,7 +297,7 @@ void LocationCompleter::indexDeleteRequested(const QModelIndex &index)
         closePopup();
     }
     else {
-        adjustPopupSize();
+        showPopup();
     }
 }
 
@@ -337,7 +339,7 @@ void LocationCompleter::showPopup()
 {
     Q_ASSERT(m_locationBar);
 
-    if (s_model->rowCount() == 0) {
+    if (!m_locationBar->hasFocus() || s_model->rowCount() == 0) {
         s_view->close();
         return;
     }
