@@ -7,47 +7,47 @@
         <translation>GreaseMonkey-installatie</translation>
     </message>
     <message>
-        <location filename="../gm_addscriptdialog.ui" line="51"/>
+        <location filename="../gm_addscriptdialog.ui" line="45"/>
         <source>&lt;h3&gt;GreaseMonkey Installation&lt;/h3&gt;</source>
         <translation>&lt;h3&gt;GreaseMonkey-installatie&lt;/h3&gt;</translation>
     </message>
     <message>
-        <location filename="../gm_addscriptdialog.ui" line="73"/>
+        <location filename="../gm_addscriptdialog.ui" line="67"/>
         <source>You are about to install this userscript into GreaseMonkey:</source>
         <translation>U staat op het punt om dit gebruikersscript te installeren in GreaseMonkey:</translation>
     </message>
     <message>
-        <location filename="../gm_addscriptdialog.ui" line="86"/>
+        <location filename="../gm_addscriptdialog.ui" line="80"/>
         <source>&lt;b&gt;You should only install scripts from sources you trust!&lt;/b&gt;</source>
         <translation>&lt;b&gt;Installeer alleen scripts van bronnen die u vertrouwt!&lt;/b&gt;</translation>
     </message>
     <message>
-        <location filename="../gm_addscriptdialog.ui" line="96"/>
+        <location filename="../gm_addscriptdialog.ui" line="90"/>
         <source>Are you sure you want to install it?</source>
         <translation>Weet u zeker dat u het script wilt installeren?</translation>
     </message>
     <message>
-        <location filename="../gm_addscriptdialog.ui" line="114"/>
+        <location filename="../gm_addscriptdialog.ui" line="108"/>
         <source>Show source code of script</source>
         <translation>Broncode van script weergeven</translation>
     </message>
     <message>
-        <location filename="../gm_addscriptdialog.cpp" line="49"/>
+        <location filename="../gm_addscriptdialog.cpp" line="50"/>
         <source>&lt;p&gt;runs at&lt;br/&gt;&lt;i&gt;%1&lt;/i&gt;&lt;/p&gt;</source>
         <translation>&lt;p&gt;wordt toegepast op&lt;br/&gt;&lt;i&gt;%1&lt;/i&gt;&lt;/p&gt;</translation>
     </message>
     <message>
-        <location filename="../gm_addscriptdialog.cpp" line="53"/>
+        <location filename="../gm_addscriptdialog.cpp" line="54"/>
         <source>&lt;p&gt;does not run at&lt;br/&gt;&lt;i&gt;%1&lt;/i&gt;&lt;/p&gt;</source>
         <translation>&lt;b&gt;wordt niet toegepast op&lt;br/&gt;&lt;i&gt;%1&lt;/i&gt;&lt;/p&gt;</translation>
     </message>
     <message>
-        <location filename="../gm_addscriptdialog.cpp" line="83"/>
+        <location filename="../gm_addscriptdialog.cpp" line="84"/>
         <source>Cannot install script</source>
         <translation>Het script kan niet worden geïnstalleerd</translation>
     </message>
     <message>
-        <location filename="../gm_addscriptdialog.cpp" line="86"/>
+        <location filename="../gm_addscriptdialog.cpp" line="87"/>
         <source>&apos;%1&apos; installed successfully</source>
         <translation>&apos;%1&apos; is succesvol geïnstalleerd</translation>
     </message>
@@ -63,12 +63,12 @@
 <context>
     <name>GM_Manager</name>
     <message>
-        <location filename="../gm_manager.cpp" line="206"/>
+        <location filename="../gm_manager.cpp" line="197"/>
         <source>GreaseMonkey</source>
         <translation>GreaseMonkey</translation>
     </message>
     <message>
-        <location filename="../gm_manager.cpp" line="270"/>
+        <location filename="../gm_manager.cpp" line="261"/>
         <source>&apos;%1&apos; is already installed</source>
         <translation>&apos;%1&apos; is al geïnstalleerd</translation>
     </message>
@@ -76,22 +76,22 @@
 <context>
     <name>GM_Notification</name>
     <message>
-        <location filename="../gm_notification.ui" line="45"/>
+        <location filename="../gm_notification.ui" line="39"/>
         <source>This script can be installed with the GreaseMonkey plugin.</source>
-        <translation>Dit script kan worden geïnstalleerd met de GreaseMonkey-plugin.</translation>
+        <translation>Dit script kan worden geïnstalleerd met de GreaseMonkey-plug-in.</translation>
     </message>
     <message>
-        <location filename="../gm_notification.ui" line="65"/>
+        <location filename="../gm_notification.ui" line="59"/>
         <source>Install</source>
         <translation>Installeren</translation>
     </message>
     <message>
-        <location filename="../gm_notification.cpp" line="50"/>
+        <location filename="../gm_notification.cpp" line="51"/>
         <source>Cannot install script</source>
         <translation>Het script kan niet worden geïnstalleerd</translation>
     </message>
     <message>
-        <location filename="../gm_notification.cpp" line="58"/>
+        <location filename="../gm_notification.cpp" line="59"/>
         <source>&apos;%1&apos; installed successfully</source>
         <translation>&apos;%1&apos; is succesvol geïnstalleerd</translation>
     </message>
@@ -104,47 +104,47 @@
         <translation>GreaseMonkey-scripts</translation>
     </message>
     <message>
-        <location filename="../settings/gm_settings.ui" line="51"/>
+        <location filename="../settings/gm_settings.ui" line="48"/>
         <source>&lt;h3&gt;GreaseMonkey Scripts&lt;/h3&gt;</source>
         <translation>&lt;h3&gt;GreaseMonkey-scripts&lt;/h3&gt;</translation>
     </message>
     <message>
-        <location filename="../settings/gm_settings.ui" line="73"/>
+        <location filename="../settings/gm_settings.ui" line="70"/>
         <source>Double clicking script will show additional information</source>
-        <translation>Door te dubbelklikken op een script zal extra informatie worden weergegeven</translation>
+        <translation>Dubbelklikken op een script zal extra informatie weergeven</translation>
     </message>
     <message>
-        <location filename="../settings/gm_settings.ui" line="153"/>
+        <location filename="../settings/gm_settings.ui" line="150"/>
         <source>More scripts can be downloaded from</source>
-        <translation>U kunt nog meer scripts verkrijgen op</translation>
+        <translation>U kunt meer scripts verkrijgen op</translation>
     </message>
     <message>
-        <location filename="../settings/gm_settings.ui" line="196"/>
+        <location filename="../settings/gm_settings.ui" line="193"/>
         <source>Open scripts directory</source>
         <translation>Scripts-map openen</translation>
     </message>
     <message>
-        <location filename="../settings/gm_settings.ui" line="203"/>
+        <location filename="../settings/gm_settings.ui" line="200"/>
         <source>New user script</source>
         <translation>Nieuw gebruikersscript</translation>
     </message>
     <message>
-        <location filename="../settings/gm_settings.cpp" line="90"/>
+        <location filename="../settings/gm_settings.cpp" line="91"/>
         <source>Remove script</source>
         <translation>Script verwijderen</translation>
     </message>
     <message>
-        <location filename="../settings/gm_settings.cpp" line="91"/>
+        <location filename="../settings/gm_settings.cpp" line="92"/>
         <source>Are you sure you want to remove &apos;%1&apos;?</source>
         <translation>Weet u zeker dat u &apos;%1&apos; wilt verwijderen?</translation>
     </message>
     <message>
-        <location filename="../settings/gm_settings.cpp" line="121"/>
+        <location filename="../settings/gm_settings.cpp" line="122"/>
         <source>Add script</source>
         <translation>Script toevoegen</translation>
     </message>
     <message>
-        <location filename="../settings/gm_settings.cpp" line="121"/>
+        <location filename="../settings/gm_settings.cpp" line="122"/>
         <source>Choose name for script:</source>
         <translation>Geef een naam op voor het script:</translation>
     </message>
@@ -184,7 +184,7 @@
     <message>
         <location filename="../settings/gm_settingsscriptinfo.ui" line="45"/>
         <source>Description:</source>
-        <translation>Beschrijving:</translation>
+        <translation>Omschrijving:</translation>
     </message>
     <message>
         <location filename="../settings/gm_settingsscriptinfo.ui" line="19"/>
